
/***************************************************************************
 *  KatanaInterface.h - Fawkes BlackBoard Interface - KatanaInterface
 *
 *  Templated created:   Thu Oct 12 10:49:19 2006
 *  Copyright  2009  Tim Niemueller
 *
 ****************************************************************************/

/*  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version. A runtime exception applies to
 *  this software (see LICENSE.GPL_WRE file mentioned below for details).
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  Read the full text in the LICENSE.GPL_WRE file in the doc directory.
 */

#ifndef _INTERFACES_KATANAINTERFACE_H_
#define _INTERFACES_KATANAINTERFACE_H_

#include <interface/interface.h>
#include <interface/message.h>
#include <interface/field_iterator.h>

namespace fawkes {

class KatanaInterface : public Interface
{
 /// @cond INTERNALS
 INTERFACE_MGMT_FRIENDS(KatanaInterface)
 /// @endcond
 public:
  /* constants */
  static const uint32_t SENSOR_IR_RIGHT_INNER_MIDDLE;
  static const uint32_t SENSOR_IR_RIGHT_INNER_FRONT;
  static const uint32_t SENSOR_RESERVED_2;
  static const uint32_t SENSOR_COND_BOTH;
  static const uint32_t SENSOR_IR_RIGHT_OUTER_FRONT;
  static const uint32_t SENSOR_IR_RIGHT_BOTTOM_FRONT;
  static const uint32_t SENSOR_FORCE_RIGHT_REAR;
  static const uint32_t SENSOR_FORCE_RIGHT_FRONT;
  static const uint32_t SENSOR_IR_LEFT_INNER_MIDDLE;
  static const uint32_t SENSOR_IR_LEFT_INNER_FRONT;
  static const uint32_t SENSOR_RESERVED_10;
  static const uint32_t SENSOR_IR_CENTER_GRIPPER;
  static const uint32_t SENSOR_IR_LEFT_OUTER_FRONT;
  static const uint32_t SENSOR_IR_LEFT_BOTTOM_FRONT;
  static const uint32_t SENSOR_FORCE_LEFT_REAR;
  static const uint32_t SENSOR_FORCE_LEFT_FRONT;
  static const uint32_t ERROR_NONE;
  static const uint32_t ERROR_UNSPECIFIC;
  static const uint32_t ERROR_CMD_START_FAILED;
  static const uint32_t ERROR_NO_SOLUTION;
  static const uint32_t ERROR_COMMUNICATION;
  static const uint32_t ERROR_MOTOR_CRASHED;

 private:
  /** Internal data storage, do NOT modify! */
  typedef struct {
    int64_t timestamp_sec;  /**< Interface Unix timestamp, seconds */
    int64_t timestamp_usec; /**< Interface Unix timestamp, micro-seconds */
    uint8_t sensor_value[16]; /**< Sensor
    values. Use SENSOR_* indexes for accessing the values. */
    float x; /**< DEPRECATED! X-Coordinate for tool position
    compared to base coordinate system. */
    float y; /**< DEPRECATED! Y-Coordinate for tool position
    compared to base coordinate system. */
    float z; /**< DEPRECATED! Z-Coordinate for tool position
    compared to base coordinate system. */
    float phi; /**< DEPRECATED! Euler angle Phi of tool orientation. */
    float theta; /**< DEPRECATED! Euler angle Theta of tool orientation. */
    float psi; /**< DEPRECATED! Euler angle Psi of tool orientation. */
    int32_t encoders[6]; /**< Encoder values of motors */
    float angles[6]; /**< Angle values of motors */
    uint32_t msgid; /**< The ID of the message that is currently being
      processed, or 0 if no message is being processed. */
    bool final; /**< True, if the last goto command has been finished,
      false if it is still running */
    uint32_t error_code; /**< Failure code set if
    final is true. 0 if no error occured, an error code from ERROR_*
    constants otherwise (or a bit-wise combination). */
    bool enabled; /**< Are motors enabled? */
    bool calibrated; /**< Has arm been calibrated? */
    uint8_t max_velocity; /**< Maximum velocity */
    uint8_t num_motors; /**< Number of motors */
  } KatanaInterface_data_t;

  KatanaInterface_data_t *data;

 public:
  /* messages */
  class StopMessage : public Message
  {
   private:
    /** Internal data storage, do NOT modify! */
    typedef struct {
      int64_t timestamp_sec;  /**< Interface Unix timestamp, seconds */
      int64_t timestamp_usec; /**< Interface Unix timestamp, micro-seconds */
    } StopMessage_data_t;

    StopMessage_data_t *data;

   public:
    StopMessage();
    ~StopMessage();

    explicit StopMessage(const StopMessage *m);
    /* Methods */
    virtual Message * clone() const;
  };

  class FlushMessage : public Message
  {
   private:
    /** Internal data storage, do NOT modify! */
    typedef struct {
      int64_t timestamp_sec;  /**< Interface Unix timestamp, seconds */
      int64_t timestamp_usec; /**< Interface Unix timestamp, micro-seconds */
    } FlushMessage_data_t;

    FlushMessage_data_t *data;

   public:
    FlushMessage();
    ~FlushMessage();

    explicit FlushMessage(const FlushMessage *m);
    /* Methods */
    virtual Message * clone() const;
  };

  class ParkMessage : public Message
  {
   private:
    /** Internal data storage, do NOT modify! */
    typedef struct {
      int64_t timestamp_sec;  /**< Interface Unix timestamp, seconds */
      int64_t timestamp_usec; /**< Interface Unix timestamp, micro-seconds */
    } ParkMessage_data_t;

    ParkMessage_data_t *data;

   public:
    ParkMessage();
    ~ParkMessage();

    explicit ParkMessage(const ParkMessage *m);
    /* Methods */
    virtual Message * clone() const;
  };

  class LinearGotoMessage : public Message
  {
   private:
    /** Internal data storage, do NOT modify! */
    typedef struct {
      int64_t timestamp_sec;  /**< Interface Unix timestamp, seconds */
      int64_t timestamp_usec; /**< Interface Unix timestamp, micro-seconds */
      float theta_error; /**< Error range of theta rotation, gives more flexibility
      for IK-solution searching. */
      float offset_xy; /**< Offset to target. Distance in m (on the way to the target) */
      bool straight; /**< Move in a straight line? */
      char trans_frame[32]; /**< tf frame-id of origin's coordinate system,
      regarding the translation */
      char rot_frame[32]; /**< tf frame-id of origin's coordinate system,
      regarding the rotation. In most cases, this is the robot's base coordinate system. */
      float x; /**< DEPRECATED! X-Coordinate for tool position
    compared to base coordinate system. */
      float y; /**< DEPRECATED! Y-Coordinate for tool position
    compared to base coordinate system. */
      float z; /**< DEPRECATED! Z-Coordinate for tool position
    compared to base coordinate system. */
      float phi; /**< DEPRECATED! Euler angle Phi of tool orientation. */
      float theta; /**< DEPRECATED! Euler angle Theta of tool orientation. */
      float psi; /**< DEPRECATED! Euler angle Psi of tool orientation. */
    } LinearGotoMessage_data_t;

    LinearGotoMessage_data_t *data;

   public:
    LinearGotoMessage(const float ini_theta_error, const float ini_offset_xy, const bool ini_straight, const char * ini_trans_frame, const char * ini_rot_frame, const float ini_x, const float ini_y, const float ini_z, const float ini_phi, const float ini_theta, const float ini_psi);
    LinearGotoMessage();
    ~LinearGotoMessage();

    explicit LinearGotoMessage(const LinearGotoMessage *m);
    /* Methods */
    float theta_error() const;
    void set_theta_error(const float new_theta_error);
    size_t maxlenof_theta_error() const;
    float offset_xy() const;
    void set_offset_xy(const float new_offset_xy);
    size_t maxlenof_offset_xy() const;
    bool is_straight() const;
    void set_straight(const bool new_straight);
    size_t maxlenof_straight() const;
    char * trans_frame() const;
    void set_trans_frame(const char * new_trans_frame);
    size_t maxlenof_trans_frame() const;
    char * rot_frame() const;
    void set_rot_frame(const char * new_rot_frame);
    size_t maxlenof_rot_frame() const;
    float x() const;
    void set_x(const float new_x);
    size_t maxlenof_x() const;
    float y() const;
    void set_y(const float new_y);
    size_t maxlenof_y() const;
    float z() const;
    void set_z(const float new_z);
    size_t maxlenof_z() const;
    float phi() const;
    void set_phi(const float new_phi);
    size_t maxlenof_phi() const;
    float theta() const;
    void set_theta(const float new_theta);
    size_t maxlenof_theta() const;
    float psi() const;
    void set_psi(const float new_psi);
    size_t maxlenof_psi() const;
    virtual Message * clone() const;
  };

  class LinearGotoKniMessage : public Message
  {
   private:
    /** Internal data storage, do NOT modify! */
    typedef struct {
      int64_t timestamp_sec;  /**< Interface Unix timestamp, seconds */
      int64_t timestamp_usec; /**< Interface Unix timestamp, micro-seconds */
      float x; /**< X-Coordinate for tool position
    compared to base libkni coordinate system. */
      float y; /**< Y-Coordinate for tool position
    compared to base libkni coordinate system. */
      float z; /**< Z-Coordinate for tool position
    compared to base libkni coordinate system. */
      float phi; /**< Euler angle Phi of tool orientation. */
      float theta; /**< Euler angle Theta of tool orientation. */
      float psi; /**< Euler angle Psi of tool orientation. */
    } LinearGotoKniMessage_data_t;

    LinearGotoKniMessage_data_t *data;

   public:
    LinearGotoKniMessage(const float ini_x, const float ini_y, const float ini_z, const float ini_phi, const float ini_theta, const float ini_psi);
    LinearGotoKniMessage();
    ~LinearGotoKniMessage();

    explicit LinearGotoKniMessage(const LinearGotoKniMessage *m);
    /* Methods */
    float x() const;
    void set_x(const float new_x);
    size_t maxlenof_x() const;
    float y() const;
    void set_y(const float new_y);
    size_t maxlenof_y() const;
    float z() const;
    void set_z(const float new_z);
    size_t maxlenof_z() const;
    float phi() const;
    void set_phi(const float new_phi);
    size_t maxlenof_phi() const;
    float theta() const;
    void set_theta(const float new_theta);
    size_t maxlenof_theta() const;
    float psi() const;
    void set_psi(const float new_psi);
    size_t maxlenof_psi() const;
    virtual Message * clone() const;
  };

  class ObjectGotoMessage : public Message
  {
   private:
    /** Internal data storage, do NOT modify! */
    typedef struct {
      int64_t timestamp_sec;  /**< Interface Unix timestamp, seconds */
      int64_t timestamp_usec; /**< Interface Unix timestamp, micro-seconds */
      char object[32]; /**< Name of object */
      float rot_x; /**< Rotation of object on its x-axis */
    } ObjectGotoMessage_data_t;

    ObjectGotoMessage_data_t *data;

   public:
    ObjectGotoMessage(const char * ini_object, const float ini_rot_x);
    ObjectGotoMessage();
    ~ObjectGotoMessage();

    explicit ObjectGotoMessage(const ObjectGotoMessage *m);
    /* Methods */
    char * object() const;
    void set_object(const char * new_object);
    size_t maxlenof_object() const;
    float rot_x() const;
    void set_rot_x(const float new_rot_x);
    size_t maxlenof_rot_x() const;
    virtual Message * clone() const;
  };

  class CalibrateMessage : public Message
  {
   private:
    /** Internal data storage, do NOT modify! */
    typedef struct {
      int64_t timestamp_sec;  /**< Interface Unix timestamp, seconds */
      int64_t timestamp_usec; /**< Interface Unix timestamp, micro-seconds */
    } CalibrateMessage_data_t;

    CalibrateMessage_data_t *data;

   public:
    CalibrateMessage();
    ~CalibrateMessage();

    explicit CalibrateMessage(const CalibrateMessage *m);
    /* Methods */
    virtual Message * clone() const;
  };

  class OpenGripperMessage : public Message
  {
   private:
    /** Internal data storage, do NOT modify! */
    typedef struct {
      int64_t timestamp_sec;  /**< Interface Unix timestamp, seconds */
      int64_t timestamp_usec; /**< Interface Unix timestamp, micro-seconds */
    } OpenGripperMessage_data_t;

    OpenGripperMessage_data_t *data;

   public:
    OpenGripperMessage();
    ~OpenGripperMessage();

    explicit OpenGripperMessage(const OpenGripperMessage *m);
    /* Methods */
    virtual Message * clone() const;
  };

  class CloseGripperMessage : public Message
  {
   private:
    /** Internal data storage, do NOT modify! */
    typedef struct {
      int64_t timestamp_sec;  /**< Interface Unix timestamp, seconds */
      int64_t timestamp_usec; /**< Interface Unix timestamp, micro-seconds */
    } CloseGripperMessage_data_t;

    CloseGripperMessage_data_t *data;

   public:
    CloseGripperMessage();
    ~CloseGripperMessage();

    explicit CloseGripperMessage(const CloseGripperMessage *m);
    /* Methods */
    virtual Message * clone() const;
  };

  class SetEnabledMessage : public Message
  {
   private:
    /** Internal data storage, do NOT modify! */
    typedef struct {
      int64_t timestamp_sec;  /**< Interface Unix timestamp, seconds */
      int64_t timestamp_usec; /**< Interface Unix timestamp, micro-seconds */
      bool enabled; /**< Are motors enabled? */
    } SetEnabledMessage_data_t;

    SetEnabledMessage_data_t *data;

   public:
    SetEnabledMessage(const bool ini_enabled);
    SetEnabledMessage();
    ~SetEnabledMessage();

    explicit SetEnabledMessage(const SetEnabledMessage *m);
    /* Methods */
    bool is_enabled() const;
    void set_enabled(const bool new_enabled);
    size_t maxlenof_enabled() const;
    virtual Message * clone() const;
  };

  class SetMaxVelocityMessage : public Message
  {
   private:
    /** Internal data storage, do NOT modify! */
    typedef struct {
      int64_t timestamp_sec;  /**< Interface Unix timestamp, seconds */
      int64_t timestamp_usec; /**< Interface Unix timestamp, micro-seconds */
      uint8_t max_velocity; /**< Maximum velocity */
    } SetMaxVelocityMessage_data_t;

    SetMaxVelocityMessage_data_t *data;

   public:
    SetMaxVelocityMessage(const uint8_t ini_max_velocity);
    SetMaxVelocityMessage();
    ~SetMaxVelocityMessage();

    explicit SetMaxVelocityMessage(const SetMaxVelocityMessage *m);
    /* Methods */
    uint8_t max_velocity() const;
    void set_max_velocity(const uint8_t new_max_velocity);
    size_t maxlenof_max_velocity() const;
    virtual Message * clone() const;
  };

  class SetPlannerParamsMessage : public Message
  {
   private:
    /** Internal data storage, do NOT modify! */
    typedef struct {
      int64_t timestamp_sec;  /**< Interface Unix timestamp, seconds */
      int64_t timestamp_usec; /**< Interface Unix timestamp, micro-seconds */
      char plannerparams[1024]; /**< Planner parameters */
      bool straight; /**< Parameters for straight movement? */
    } SetPlannerParamsMessage_data_t;

    SetPlannerParamsMessage_data_t *data;

   public:
    SetPlannerParamsMessage(const char * ini_plannerparams, const bool ini_straight);
    SetPlannerParamsMessage();
    ~SetPlannerParamsMessage();

    explicit SetPlannerParamsMessage(const SetPlannerParamsMessage *m);
    /* Methods */
    char * plannerparams() const;
    void set_plannerparams(const char * new_plannerparams);
    size_t maxlenof_plannerparams() const;
    bool is_straight() const;
    void set_straight(const bool new_straight);
    size_t maxlenof_straight() const;
    virtual Message * clone() const;
  };

  class SetMotorEncoderMessage : public Message
  {
   private:
    /** Internal data storage, do NOT modify! */
    typedef struct {
      int64_t timestamp_sec;  /**< Interface Unix timestamp, seconds */
      int64_t timestamp_usec; /**< Interface Unix timestamp, micro-seconds */
      uint32_t nr; /**< Motor number */
      uint32_t enc; /**< Encoder value */
    } SetMotorEncoderMessage_data_t;

    SetMotorEncoderMessage_data_t *data;

   public:
    SetMotorEncoderMessage(const uint32_t ini_nr, const uint32_t ini_enc);
    SetMotorEncoderMessage();
    ~SetMotorEncoderMessage();

    explicit SetMotorEncoderMessage(const SetMotorEncoderMessage *m);
    /* Methods */
    uint32_t nr() const;
    void set_nr(const uint32_t new_nr);
    size_t maxlenof_nr() const;
    uint32_t enc() const;
    void set_enc(const uint32_t new_enc);
    size_t maxlenof_enc() const;
    virtual Message * clone() const;
  };

  class MoveMotorEncoderMessage : public Message
  {
   private:
    /** Internal data storage, do NOT modify! */
    typedef struct {
      int64_t timestamp_sec;  /**< Interface Unix timestamp, seconds */
      int64_t timestamp_usec; /**< Interface Unix timestamp, micro-seconds */
      uint32_t nr; /**< Motor number */
      uint32_t enc; /**< Encoder value */
    } MoveMotorEncoderMessage_data_t;

    MoveMotorEncoderMessage_data_t *data;

   public:
    MoveMotorEncoderMessage(const uint32_t ini_nr, const uint32_t ini_enc);
    MoveMotorEncoderMessage();
    ~MoveMotorEncoderMessage();

    explicit MoveMotorEncoderMessage(const MoveMotorEncoderMessage *m);
    /* Methods */
    uint32_t nr() const;
    void set_nr(const uint32_t new_nr);
    size_t maxlenof_nr() const;
    uint32_t enc() const;
    void set_enc(const uint32_t new_enc);
    size_t maxlenof_enc() const;
    virtual Message * clone() const;
  };

  class SetMotorAngleMessage : public Message
  {
   private:
    /** Internal data storage, do NOT modify! */
    typedef struct {
      int64_t timestamp_sec;  /**< Interface Unix timestamp, seconds */
      int64_t timestamp_usec; /**< Interface Unix timestamp, micro-seconds */
      uint32_t nr; /**< Motor number */
      float angle; /**< Angle value (positive: increase; negative: decrease) */
    } SetMotorAngleMessage_data_t;

    SetMotorAngleMessage_data_t *data;

   public:
    SetMotorAngleMessage(const uint32_t ini_nr, const float ini_angle);
    SetMotorAngleMessage();
    ~SetMotorAngleMessage();

    explicit SetMotorAngleMessage(const SetMotorAngleMessage *m);
    /* Methods */
    uint32_t nr() const;
    void set_nr(const uint32_t new_nr);
    size_t maxlenof_nr() const;
    float angle() const;
    void set_angle(const float new_angle);
    size_t maxlenof_angle() const;
    virtual Message * clone() const;
  };

  class MoveMotorAngleMessage : public Message
  {
   private:
    /** Internal data storage, do NOT modify! */
    typedef struct {
      int64_t timestamp_sec;  /**< Interface Unix timestamp, seconds */
      int64_t timestamp_usec; /**< Interface Unix timestamp, micro-seconds */
      uint32_t nr; /**< Motor number */
      float angle; /**< Angle value (positive: increase; negative: decrease) */
    } MoveMotorAngleMessage_data_t;

    MoveMotorAngleMessage_data_t *data;

   public:
    MoveMotorAngleMessage(const uint32_t ini_nr, const float ini_angle);
    MoveMotorAngleMessage();
    ~MoveMotorAngleMessage();

    explicit MoveMotorAngleMessage(const MoveMotorAngleMessage *m);
    /* Methods */
    uint32_t nr() const;
    void set_nr(const uint32_t new_nr);
    size_t maxlenof_nr() const;
    float angle() const;
    void set_angle(const float new_angle);
    size_t maxlenof_angle() const;
    virtual Message * clone() const;
  };

  virtual bool message_valid(const Message *message) const;
 private:
  KatanaInterface();
  ~KatanaInterface();

 public:
  /* Methods */
  uint8_t * sensor_value() const;
  uint8_t sensor_value(unsigned int index) const;
  void set_sensor_value(unsigned int index, const uint8_t new_sensor_value);
  void set_sensor_value(const uint8_t * new_sensor_value);
  size_t maxlenof_sensor_value() const;
  float x() const;
  void set_x(const float new_x);
  size_t maxlenof_x() const;
  float y() const;
  void set_y(const float new_y);
  size_t maxlenof_y() const;
  float z() const;
  void set_z(const float new_z);
  size_t maxlenof_z() const;
  float phi() const;
  void set_phi(const float new_phi);
  size_t maxlenof_phi() const;
  float theta() const;
  void set_theta(const float new_theta);
  size_t maxlenof_theta() const;
  float psi() const;
  void set_psi(const float new_psi);
  size_t maxlenof_psi() const;
  int32_t * encoders() const;
  int32_t encoders(unsigned int index) const;
  void set_encoders(unsigned int index, const int32_t new_encoders);
  void set_encoders(const int32_t * new_encoders);
  size_t maxlenof_encoders() const;
  float * angles() const;
  float angles(unsigned int index) const;
  void set_angles(unsigned int index, const float new_angles);
  void set_angles(const float * new_angles);
  size_t maxlenof_angles() const;
  uint32_t msgid() const;
  void set_msgid(const uint32_t new_msgid);
  size_t maxlenof_msgid() const;
  bool is_final() const;
  void set_final(const bool new_final);
  size_t maxlenof_final() const;
  uint32_t error_code() const;
  void set_error_code(const uint32_t new_error_code);
  size_t maxlenof_error_code() const;
  bool is_enabled() const;
  void set_enabled(const bool new_enabled);
  size_t maxlenof_enabled() const;
  bool is_calibrated() const;
  void set_calibrated(const bool new_calibrated);
  size_t maxlenof_calibrated() const;
  uint8_t max_velocity() const;
  void set_max_velocity(const uint8_t new_max_velocity);
  size_t maxlenof_max_velocity() const;
  uint8_t num_motors() const;
  void set_num_motors(const uint8_t new_num_motors);
  size_t maxlenof_num_motors() const;
  virtual Message * create_message(const char *type) const;

  virtual void copy_values(const Interface *other);
  virtual const char * enum_tostring(const char *enumtype, int val) const;

};

} // end namespace fawkes

#endif
